@@ -54,6 +54,8 @@
 
 /* Needed for GENA */
 #include "gena.h"
+#include "gena_ctrlpt.h"
+#include "gena_device.h"
 #include "gena_sids.h"
 #include "miniserver.h"
 #include "service_table.h"
@@ -79,11 +81,6 @@
 /*! This structure is for virtual directory callbacks */
 struct VirtualDirCallbacks virtualDirCallback;
 
-#ifdef INCLUDE_CLIENT_APIS
-/* Mutex to synchronize the subscription handling at the client side. */
-std::mutex GlobalClientSubscribeMutex;
-#endif /* INCLUDE_CLIENT_APIS */
-
 // Mutex to synchronize handles (root device or control point
 // handle). This used to be an rwlock but this was probably not worth
 // the trouble given the small expected contention level
@@ -176,19 +173,19 @@ Upnp_SID gUpnpSdkNLSuuid;
 std::string apiFirstIPV4Str()
 {
     if (g_netifs.empty())
-        return std::string();
+        return {};
     const auto a = g_netifs.begin()->firstipv4addr();
     if (nullptr == a)
-        return std::string();
+        return {};
     return a->straddr();
 }
 std::string apiFirstIPV6Str()
 {
     if (g_netifs.empty())
-        return std::string();
+        return {};
     const auto a = g_netifs.begin()->firstipv6addr();
     if (nullptr == a)
-        return std::string();
+        return {};
     return a->straddr();
 }
 int apiFirstIPV6Index()
@@ -747,7 +744,6 @@ void PrintThreadPoolStats(
 static UPNP_INLINE void PrintThreadPoolStats(
     ThreadPool *, const char *, int, const char *)
 {
-    return;
 }
 #endif /* DEBUG */
 
@@ -806,6 +802,12 @@ EXPORT_SPEC int UpnpSetWebRequestHostValidateCallback(
     g_hostvalidatecookie = cookie;
     return UPNP_E_SUCCESS;
 }
+EXPORT_SPEC int UpnpSetHostValidateCallback(
+    WebCallback_HostValidate callback, void *cookie)
+{
+    return UpnpSetWebRequestHostValidateCallback(callback, cookie);
+}
+
 
 EXPORT_SPEC std::string UpnpGetUrlHostPortForClient(const struct sockaddr_storage* clsock)
 {
@@ -814,7 +816,7 @@ EXPORT_SPEC std::string UpnpGetUrlHostPortForClient(const struct sockaddr_storag
     const NetIF::Interface *itf =
         NetIF::Interfaces::interfaceForAddress(claddr, g_netifs, hostaddr);
     if (nullptr == itf) {
-        return std::string();
+        return {};
     }
     int port = 0;
     std::string prefix;
@@ -827,7 +829,7 @@ EXPORT_SPEC std::string UpnpGetUrlHostPortForClient(const struct sockaddr_storag
         port = UpnpGetServerPort6();
             break;
     default:
-        return std::string();
+        return {};
     }
 
     return prefix + hostaddr.straddr() + (prefix.empty() ? "" : "]") + ":" + lltodecstr(port);
@@ -1200,9 +1202,6 @@ int UpnpUnRegisterClient(UpnpClient_Handle Hnd)
         return UPNP_E_INVALID_HANDLE;
     }
     /* clean up search list */
-    for (auto & it : HInfo->SsdpSearchList) {
-        delete it;
-    }
     HInfo->SsdpSearchList.clear();
 
     FreeHandle(Hnd);
@@ -1432,11 +1431,13 @@ int UpnpSendAdvertisement(UpnpDevice_Handle Hnd, int Exp)
     return UpnpSendAdvertisementLowPower(Hnd, Exp, -1, -1, -1);
 }
 
-void thread_autoadvertise(void *input)
+void* thread_autoadvertise(void *input)
 {
     auto event = static_cast<upnp_timeout *>(input);
 
     UpnpSendAdvertisement(event->handle, *(static_cast<int *>(event->Event)));
+
+    return nullptr;
 }
 
 int UpnpSendAdvertisementLowPower(
@@ -1490,19 +1491,14 @@ int UpnpSendAdvertisementLowPower(
     }
 
 #ifdef SSDP_PACKET_DISTRIBUTE
-    retVal = gTimerThread->schedule(
-        TimerThread::SHORT_TERM, TimerThread::REL_SEC,
-        ((Exp / 2) - (AUTO_ADVERTISEMENT_TIME)),
-         &(adEvent->eventId),
-        reinterpret_cast<start_routine>(thread_autoadvertise), adEvent,
-        reinterpret_cast<ThreadPool::free_routine>(free_upnp_timeout));
+    time_t thetime = ((Exp / 2) - (AUTO_ADVERTISEMENT_TIME));
 #else
-    retVal = gTimerThread->schedule(
-        TimerThread::SHORT_TERM, TimerThread::REL_SEC,
-        Exp - AUTO_ADVERTISEMENT_TIME, &(adEvent->eventId),
-        (start_routine)thread_autoadvertise, adEvent,
-        (ThreadPool::free_routine)free_upnp_timeout);
+    time_t thetime = Exp - AUTO_ADVERTISEMENT_TIME;
 #endif
+
+    retVal = gTimerThread->schedule(
+        TimerThread::SHORT_TERM, TimerThread::REL_SEC, thetime, &adEvent->eventId,
+        thread_autoadvertise, adEvent,reinterpret_cast<ThreadPool::free_routine>(free_upnp_timeout));
     if (retVal != UPNP_E_SUCCESS) {
         HandleUnlock();
         free_upnp_timeout(adEvent);
