@@ -41,6 +41,7 @@
 #include <string>
 #include <map>
 #include <sstream>
+#include <algorithm>
 
 #include "gena.h"
 #include "httputils.h"
@@ -60,8 +61,27 @@
 #define XMLPARSERTP PicoXMLParser
 #endif
 
+#define UPNP_NOPE (Upnp_LogLevel)(UPNP_ALL+1)
+
 extern TimerThread *gTimerThread;
 
+/* Mutex to synchronize client subscription processing */
+std::mutex GlobalClientSubscribeMutex;
+
+#define SubscribeLock() do {                                            \
+    UpnpPrintf(UPNP_NOPE, GENA, __FILE__, __LINE__, "Trying Subscribe Lock\n"); \
+    GlobalClientSubscribeMutex.lock();                                  \
+    UpnpPrintf(UPNP_NOPE, GENA, __FILE__, __LINE__, "Subscribe Lock\n"); \
+    } while(0)
+
+
+#define SubscribeUnlock() do {                                          \
+    UpnpPrintf(UPNP_NOPE, GENA, __FILE__,__LINE__, "Trying Subscribe UnLock\n"); \
+    GlobalClientSubscribeMutex.unlock();                                \
+    UpnpPrintf(UPNP_NOPE, GENA, __FILE__, __LINE__, "Subscribe UnLock\n"); \
+    } while(0)
+
+
 static void clientCancelRenew(ClientSubscription *sub)
 {
     if (nullptr == sub) {
@@ -69,7 +89,7 @@ static void clientCancelRenew(ClientSubscription *sub)
     }
     int renewEventId = sub->renewEventId;
     sub->renewEventId = -1;
-    sub->actualSID.clear();
+    sub->SID.clear();
     sub->eventURL.clear();
     if (renewEventId != -1) {
         gTimerThread->remove(renewEventId);
@@ -115,7 +135,7 @@ static void *thread_autorenewsubscription(
         if (GetHandleInfo(event->handle, &handle_info) != HND_CLIENT) {
             HandleUnlock();
             free_upnp_timeout(event);
-            goto end_function;
+            return nullptr;
         }
 
         /* make callback */
@@ -123,8 +143,6 @@ static void *thread_autorenewsubscription(
         HandleUnlock();
         callback_fun(eventType, event->Event, handle_info->Cookie);
     }
-
-end_function:
     return nullptr;
 }
 
@@ -143,30 +161,20 @@ static int ScheduleGenaAutoRenew(
     /*! [in] Subscription being renewed. */
     ClientSubscription *sub)
 {
-    struct Upnp_Event_Subscribe *RenewEventStruct = nullptr;
-    upnp_timeout *RenewEvent = nullptr;
-    int return_code = GENA_SUCCESS;
     const std::string& tmpSID = sub->SID;
     const std::string& tmpEventURL = sub->eventURL;
 
     if (TimeOut == UPNP_INFINITE) {
-        return_code = GENA_SUCCESS;
-        goto end_function;
+        return GENA_SUCCESS;
     }
 
-    RenewEventStruct = static_cast<struct Upnp_Event_Subscribe *>(malloc(
-        sizeof(struct Upnp_Event_Subscribe)));
+    auto RenewEventStruct =
+        static_cast<struct Upnp_Event_Subscribe *>(malloc(sizeof(struct Upnp_Event_Subscribe)));
     if (RenewEventStruct == nullptr) {
-        return_code = UPNP_E_OUTOF_MEMORY;
-        goto end_function;
+        return UPNP_E_OUTOF_MEMORY;
     }
 
-    RenewEvent =  new upnp_timeout;
-    if (nullptr == RenewEvent) {
-        free(RenewEventStruct);
-        return_code = UPNP_E_OUTOF_MEMORY;
-        goto end_function;
-    }
+    auto RenewEvent =  new upnp_timeout;
 
     /* schedule expire event */
     *RenewEventStruct = {};
@@ -179,28 +187,23 @@ static int ScheduleGenaAutoRenew(
     RenewEvent->Event = RenewEventStruct;
 
     /* Schedule the job */
-    return_code = gTimerThread->schedule(
+    int return_code = gTimerThread->schedule(
         TimerThread::SHORT_TERM, TimerThread::REL_SEC, TimeOut - AUTO_RENEW_TIME,
         &(RenewEvent->eventId), thread_autorenewsubscription, RenewEvent,
         reinterpret_cast<ThreadPool::free_routine>(free_upnp_timeout));
 
     if (return_code != UPNP_E_SUCCESS) {
         free_upnp_timeout(RenewEvent);
-        goto end_function;
+        return return_code;
     }
 
     sub->renewEventId = RenewEvent->eventId;
-
-    return_code = GENA_SUCCESS;
-
-end_function:
-    return return_code;
+    return GENA_SUCCESS;
 }
 
 
 /*!
- * \brief Sends the UNSUBCRIBE gena request and recieves the response from the
- *     device and returns it as a parameter.
+ * \brief Sends the UNSUBCRIBE gena request
  *
  * \returns 0 if successful, otherwise returns the appropriate error code.
  */
@@ -213,6 +216,9 @@ static int gena_unsubscribe(
     int return_code;
     uri_type dest_url;
 
+    UpnpPrintf(UPNP_ALL,GENA,__FILE__,__LINE__, "gena_unsubscribe: SID [%s] url [%s]\n",
+               sid.c_str(), url.c_str());
+
     /* parse url */
     return_code = http_FixStrUrl(url, &dest_url);
     if (return_code != 0) {
@@ -335,14 +341,14 @@ static int gena_subscribe(
                    "Could not find the interface for the destination address\n");
         return UPNP_E_SOCKET_CONNECT;
     }
-        
+       
     CurlGuard hdls;
     std::map<std::string, std::string> http_headers;
     char curlerrormessage[CURL_ERROR_SIZE];
     
     hdls.htalk = curl_easy_init();
     curl_easy_setopt(hdls.htalk, CURLOPT_ERRORBUFFER, curlerrormessage);
-    curl_easy_setopt(hdls.htalk, CURLOPT_WRITEFUNCTION,write_callback_null_curl);
+    curl_easy_setopt(hdls.htalk, CURLOPT_WRITEFUNCTION, write_callback_null_curl);
     curl_easy_setopt(hdls.htalk, CURLOPT_CUSTOMREQUEST, "SUBSCRIBE");
     curl_easy_setopt(hdls.htalk, CURLOPT_URL, urlforcurl.c_str());
     curl_easy_setopt(hdls.htalk, CURLOPT_TIMEOUT, HTTP_DEFAULT_TIMEOUT);
@@ -352,12 +358,16 @@ static int gena_subscribe(
         std::string cbheader{"CALLBACK: <"};
         cbheader += myCallbackUrl(myaddr) + "/>";
         hdls.hlist = curl_slist_append(hdls.hlist, cbheader.c_str());
-        UpnpPrintf(UPNP_DEBUG, GENA, __FILE__, __LINE__,
-                   "gena_subscribe: callback: %s\n", cbheader.c_str());
         hdls.hlist = curl_slist_append(hdls.hlist, "NT: upnp:event");
+        UpnpPrintf(UPNP_ALL,GENA,__FILE__,__LINE__,
+                   "gena_subscribe(init.): url [%s] cb [%s] timeout [%s]\n",
+                   urlforcurl.c_str(), myCallbackUrl(myaddr).c_str(), timostr.str().c_str());
     } else {
         hdls.hlist = curl_slist_append(
             hdls.hlist, (std::string("SID: ") + renewal_sid).c_str());
+        UpnpPrintf(UPNP_ALL,GENA,__FILE__,__LINE__,
+                   "gena_subscribe(renew): SID [%s] url [%s] timeout [%s]\n",
+                   renewal_sid.c_str(), urlforcurl.c_str(), timostr.str().c_str());
     }
     hdls.hlist = curl_slist_append(
         hdls.hlist, (std::string("TIMEOUT: Second-") + timostr.str()).c_str());
@@ -375,6 +385,8 @@ static int gena_subscribe(
     long http_status;
     curl_easy_getinfo (hdls.htalk, CURLINFO_RESPONSE_CODE, &http_status);
     if (http_status != HTTP_OK) {
+        UpnpPrintf(UPNP_DEBUG,GENA,__FILE__,__LINE__, "gena_subscribe: HTTP status %d\n",
+                   int(http_status));
         return UPNP_E_SUBSCRIBE_UNACCEPTED;
     }
 
@@ -382,16 +394,20 @@ static int gena_subscribe(
     const auto itsid = http_headers.find("sid");
     const auto ittimeout = http_headers.find("timeout");
     if (itsid == http_headers.end() || ittimeout == http_headers.end()) {
+        UpnpPrintf(UPNP_DEBUG,GENA,__FILE__,__LINE__, "Subscribe error: no SID in answer\n");
         return UPNP_E_BAD_RESPONSE;
     }
 
     /* save timeout */
     if (!timeout_header_value(http_headers, timeout)) {
+        UpnpPrintf(UPNP_DEBUG,GENA,__FILE__,__LINE__, "Subscribe error: no timeout in answer\n");
         return UPNP_E_BAD_RESPONSE;
     }
 
     /* save SID */
     *sid = itsid->second;
+    UpnpPrintf(UPNP_ALL,GENA,__FILE__,__LINE__, "gena_subscribe ok: SID [%s] timeout %d\n",
+               itsid->second.c_str(), *timeout);
     
     return UPNP_E_SUCCESS;
 }
@@ -414,11 +430,12 @@ int genaUnregisterClient(UpnpClient_Handle client_handle)
             break;
         }
         ClientSubscription sub_copy = handle_info->ClientSubList.front();
-        RemoveClientSubClientSID(handle_info->ClientSubList, sub_copy.SID);
+        handle_info->ClientSubList.remove_if(
+            [sub_copy] (const ClientSubscription& e) {return e.SID == sub_copy.SID;});
 
         HandleUnlock();
 
-        gena_unsubscribe(sub_copy.eventURL, sub_copy.actualSID);
+        gena_unsubscribe(sub_copy.eventURL, sub_copy.SID);
         clientCancelRenew(&sub_copy);
     }
 
@@ -429,45 +446,37 @@ int genaUnregisterClient(UpnpClient_Handle client_handle)
 }
 
 
-int genaUnSubscribe(
-    UpnpClient_Handle client_handle,
-    const std::string& in_sid)
+int genaUnSubscribe(UpnpClient_Handle client_handle, const std::string& in_sid)
 {
-    ClientSubscription *sub = nullptr;
-    int return_code = GENA_SUCCESS;
-    struct Handle_Info *handle_info;
-    ClientSubscription sub_copy;
-
     /* validate handle and sid */
     HandleLock();
+    struct Handle_Info *handle_info;
     if (GetHandleInfo(client_handle, &handle_info) != HND_CLIENT) {
         HandleUnlock();
-        return_code = GENA_E_BAD_HANDLE;
-        goto exit_function;
+        return GENA_E_BAD_HANDLE;
     }
-    sub = GetClientSubClientSID(handle_info->ClientSubList, in_sid);
-    if (nullptr == sub) {
+    auto sub = std::find_if(handle_info->ClientSubList.begin(), handle_info->ClientSubList.end(),
+                            [in_sid](const ClientSubscription& e){return e.SID == in_sid;});
+    if (handle_info->ClientSubList.end() == sub) {
         HandleUnlock();
-        return_code = GENA_E_BAD_SID;
-        goto exit_function;
+        return GENA_E_BAD_SID;
     }
-    sub_copy = *sub;
+    auto sub_copy = *sub;
     HandleUnlock();
 
-    return_code = gena_unsubscribe(sub_copy.eventURL, sub_copy.actualSID);
+    gena_unsubscribe(sub_copy.eventURL, sub_copy.SID);
     clientCancelRenew(&sub_copy);
 
     HandleLock();
     if (GetHandleInfo(client_handle, &handle_info) != HND_CLIENT) {
         HandleUnlock();
-        return_code = GENA_E_BAD_HANDLE;
-        goto exit_function;
+        return GENA_E_BAD_HANDLE;
     }
-    RemoveClientSubClientSID(handle_info->ClientSubList, in_sid);
+    handle_info->ClientSubList.remove_if(
+        [in_sid] (const ClientSubscription& e) {return e.SID == in_sid;});
     HandleUnlock();
 
-exit_function:
-    return return_code;
+    return GENA_SUCCESS;
 }
 
 
@@ -479,12 +488,10 @@ int genaSubscribe(
 {
     int return_code = GENA_SUCCESS;
     ClientSubscription newSubscription;
-    std::string ActualSID;
+    std::string SID;
     std::string EventURL;
     struct Handle_Info *handle_info;
 
-    UpnpPrintf(UPNP_DEBUG, GENA, __FILE__, __LINE__, "genaSubscribe\n");
-
     out_sid->clear();
 
     HandleReadLock();
@@ -498,11 +505,11 @@ int genaSubscribe(
 
     /* subscribe */
     SubscribeLock();
-    return_code = gena_subscribe(PublisherURL,TimeOut,std::string(), &ActualSID);
+    return_code = gena_subscribe(PublisherURL, TimeOut, std::string(), &SID);
     HandleLock();
     if (return_code != UPNP_E_SUCCESS) {
         UpnpPrintf(UPNP_ERROR, GENA, __FILE__, __LINE__,
-                   "genSubscribe: subscribe error, return %d\n", return_code);
+                   "genaSubscribe: subscribe error, return %d\n", return_code);
         goto error_handler;
     }
 
@@ -511,15 +518,11 @@ int genaSubscribe(
         goto error_handler;
     }
 
-    /* generate client SID */
-    out_sid->assign(std::string("uuid:") + gena_sid_uuid());
-
     /* create event url */
     EventURL = PublisherURL;
-
     newSubscription.renewEventId = -1;
-    newSubscription.SID = *out_sid;
-    newSubscription.actualSID = ActualSID;
+    newSubscription.SID = SID;
+    out_sid->assign(SID);
     newSubscription.eventURL = EventURL;
     handle_info->ClientSubList.push_front(newSubscription);
 
@@ -540,74 +543,70 @@ int genaRenewSubscription(
     const std::string& in_sid,
     int *TimeOut)
 {
-    int return_code = GENA_SUCCESS;
-    ClientSubscription *sub = nullptr;
-    ClientSubscription sub_copy;
-    struct Handle_Info *handle_info;
-    std::string ActualSID;
-
     HandleLock();
 
     /* validate handle and sid */
+    struct Handle_Info *handle_info;
     if (GetHandleInfo(client_handle, &handle_info) != HND_CLIENT) {
         HandleUnlock();
-        return_code = GENA_E_BAD_HANDLE;
-        goto exit_function;
+        return GENA_E_BAD_HANDLE;
     }
 
-    sub = GetClientSubClientSID(handle_info->ClientSubList, in_sid);
-    if (sub == nullptr) {
+    auto sub = std::find_if(handle_info->ClientSubList.begin(), handle_info->ClientSubList.end(),
+                            [in_sid](const ClientSubscription& e){return e.SID == in_sid;});
+    if (handle_info->ClientSubList.end() == sub) {
         HandleUnlock();
-        return_code = GENA_E_BAD_SID;
-        goto exit_function;
+        return GENA_E_BAD_SID;
     }
+
     /* remove old events */
     gTimerThread->remove(sub->renewEventId);
 
     sub->renewEventId = -1;
-    sub_copy = *sub;
+    ClientSubscription sub_copy = *sub;
 
     HandleUnlock();
 
-    return_code = gena_subscribe(sub_copy.eventURL, TimeOut, sub_copy.actualSID, &ActualSID);
+    std::string SID;
+    int return_code = gena_subscribe(sub_copy.eventURL, TimeOut, sub_copy.SID, &SID);
 
     HandleLock();
 
     if (GetHandleInfo(client_handle, &handle_info) != HND_CLIENT) {
         HandleUnlock();
-        return_code = GENA_E_BAD_HANDLE;
-        goto exit_function;
+        return GENA_E_BAD_HANDLE;
     }
 
     if (return_code != UPNP_E_SUCCESS) {
         /* network failure (remove client sub) */
-        RemoveClientSubClientSID(handle_info->ClientSubList, in_sid);
+        handle_info->ClientSubList.remove_if(
+            [in_sid] (const ClientSubscription& e) {return e.SID == in_sid;});
         clientCancelRenew(&sub_copy);
         HandleUnlock();
-        goto exit_function;
+        return return_code;
     }
 
     /* get subscription */
-    sub = GetClientSubClientSID(handle_info->ClientSubList, in_sid);
-    if (sub == nullptr) {
+    sub = std::find_if(handle_info->ClientSubList.begin(), handle_info->ClientSubList.end(),
+                       [in_sid](const ClientSubscription& e){return e.SID == in_sid;});
+    if (handle_info->ClientSubList.end() == sub) {
         clientCancelRenew(&sub_copy);
         HandleUnlock();
-        return_code = GENA_E_BAD_SID;
-        goto exit_function;
+        return GENA_E_BAD_SID;
     }
 
-    /* store actual sid */
-    sub->actualSID = ActualSID;
+    /* Remember SID */
+    sub->SID = SID;
 
     /* start renew subscription timer */
-    return_code = ScheduleGenaAutoRenew(client_handle, *TimeOut, sub);
+    return_code = ScheduleGenaAutoRenew(client_handle, *TimeOut, &*sub);
     if (return_code != GENA_SUCCESS) {
-        RemoveClientSubClientSID(handle_info->ClientSubList, sub->SID);
+        handle_info->ClientSubList.remove_if(
+            [sub] (const ClientSubscription& e) {return e.SID == sub->SID;});
     }
     clientCancelRenew(&sub_copy);
-    HandleUnlock();
 
-exit_function:
+    HandleUnlock();
     return return_code;
 }
 
@@ -646,17 +645,7 @@ private:
 
 void gena_process_notification_event(MHDTransaction *mhdt)
 {
-    struct Upnp_Event event_struct;
-    int eventKey;
-    ClientSubscription *subscription = nullptr;
-    struct Handle_Info *handle_info;
-    void *cookie;
-    Upnp_FunPtr callback;
-    UpnpClient_Handle client_handle;
-    std::string tmpSID;
-
-    UpnpPrintf(UPNP_DEBUG, GENA, __FILE__, __LINE__,
-               "gena_process_notification_event\n");
+    UpnpPrintf(UPNP_ALL, GENA, __FILE__, __LINE__, "gena_process_notification_event\n");
     
     auto itsid = mhdt->headers.find("sid");
     /* get SID */
@@ -675,6 +664,7 @@ void gena_process_notification_event(MHDTransaction *mhdt)
         return;
     }
     char cb[2];
+    int eventKey;
     if (sscanf(itseq->second.c_str(), "%d%1c", &eventKey, cb) != 1) {
         http_SendStatusResponse(mhdt, HTTP_BAD_REQUEST);
         UpnpPrintf(UPNP_DEBUG,GENA,__FILE__,__LINE__, "gena_process_notification_event: bad seq\n");
@@ -717,6 +707,8 @@ void gena_process_notification_event(MHDTransaction *mhdt)
     HandleLock();
 
     /* get client info */
+    struct Handle_Info *handle_info;
+    UpnpClient_Handle client_handle;
     if (GetClientHandleInfo(&client_handle, &handle_info) != HND_CLIENT) {
         http_SendStatusResponse(mhdt, HTTP_PRECONDITION_FAILED);
         HandleUnlock();
@@ -724,8 +716,10 @@ void gena_process_notification_event(MHDTransaction *mhdt)
     }
 
     /* get subscription based on SID */
-    subscription = GetClientSubActualSID(handle_info->ClientSubList, sid);
-    if (subscription == nullptr) {
+    auto subscription = std::find_if(
+        handle_info->ClientSubList.begin(), handle_info->ClientSubList.end(),
+        [sid](const ClientSubscription& e){return e.SID == sid;});
+    if (handle_info->ClientSubList.end() == subscription) {
         if (eventKey == 0) {
             /* wait until we've finished processing a subscription  */
             /*   (if we are in the middle) */
@@ -747,8 +741,10 @@ void gena_process_notification_event(MHDTransaction *mhdt)
                 return;
             }
 
-            subscription = GetClientSubActualSID(handle_info->ClientSubList,sid);
-            if (subscription == nullptr) {
+            subscription = std::find_if(
+                handle_info->ClientSubList.begin(), handle_info->ClientSubList.end(),
+                [sid](const ClientSubscription& e){return e.SID == sid;});
+            if (handle_info->ClientSubList.end() == subscription) {
                 http_SendStatusResponse(mhdt, HTTP_PRECONDITION_FAILED);
                 SubscribeUnlock();
                 HandleUnlock();
@@ -757,6 +753,10 @@ void gena_process_notification_event(MHDTransaction *mhdt)
 
             SubscribeUnlock();
         } else {
+            UpnpPrintf(UPNP_DEBUG, GENA, __FILE__, __LINE__,
+                       "gena_process_notification_event: could not find subscription "
+                       "but event key not 0 (%d)\n", eventKey);
+            
             http_SendStatusResponse(mhdt, HTTP_PRECONDITION_FAILED);
             HandleUnlock();
             return;
@@ -767,15 +767,16 @@ void gena_process_notification_event(MHDTransaction *mhdt)
     http_SendStatusResponse(mhdt, HTTP_OK);
 
     /* fill event struct */
-    tmpSID = subscription->SID;
+    auto tmpSID = subscription->SID;
+    struct Upnp_Event event_struct;
     memset(event_struct.Sid, 0, sizeof(event_struct.Sid));
     upnp_strlcpy(event_struct.Sid, tmpSID, sizeof(event_struct.Sid));
     event_struct.EventKey = eventKey;
     event_struct.ChangedVariables = propset;
 
     /* copy callback */
-    callback = handle_info->Callback;
-    cookie = handle_info->Cookie;
+    auto callback = handle_info->Callback;
+    auto cookie = handle_info->Cookie;
 
     HandleUnlock();
 
