@@ -155,7 +155,7 @@ void ssdp_handle_device_request(SSDPPacketParser& parser,
         UpnpPrintf(UPNP_DEBUG, API, __FILE__, __LINE__,
                    "ServiceType =  %s\n", event.ServiceType.c_str());
 
-        SsdpSearchReply *threadArg = new SsdpSearchReply;
+        auto threadArg = new SsdpSearchReply;
         if (threadArg == nullptr)
             return;
         threadArg->handle = handle;
@@ -305,7 +305,7 @@ static SOCKET createReplySocket6(
 {
     return INVALID_SOCKET;
 }
-#endif // ! ENABLE_IPV6
+#endif // UPNP_ENABLE_IPV6
 
 // Set the UPnP predefined multicast destination addresses
 static bool ssdpMcastAddr(struct sockaddr_storage& ss, int AddressFamily)
