@@ -68,12 +68,12 @@
 #ifndef IN6_IS_ADDR_GLOBAL
 #define IN6_IS_ADDR_GLOBAL(a) \
     ((((__const uint32_t *) (a))[0] & htonl(0x70000000)) == htonl (0x20000000))
-#endif /* IS ADDR GLOBAL */
+#endif /* IN6_IS_ADDR_GLOBAL */
 
 #ifndef IN6_IS_ADDR_ULA
 #define IN6_IS_ADDR_ULA(a) \
         ((((__const uint32_t *) (a))[0] & htonl(0xfe000000)) == htonl (0xfc000000))
-#endif /* IS ADDR ULA */
+#endif /* IN6_IS_ADDR_ULA */
 
 
 namespace NetIF {
@@ -104,13 +104,13 @@ public:
 
 IPAddr::IPAddr()
 {
-    m = new Internal;
+    m.reset(new Internal);
     m->saddr = reinterpret_cast<struct sockaddr*>(&m->address);
 }
 
 IPAddr::IPAddr(const IPAddr& o)
 {
-    m = new Internal;
+    m.reset(new Internal);
     *m = *(o.m);
     m->saddr = reinterpret_cast<struct sockaddr*>(&m->address);
 }
@@ -118,8 +118,7 @@ IPAddr::IPAddr(const IPAddr& o)
 IPAddr& IPAddr::operator=(const IPAddr& o)
 {
     if (&o != this) {
-        delete m;
-        m = new Internal;
+        m.reset(new Internal);
         *m = *(o.m);
         m->saddr = reinterpret_cast<struct sockaddr*>(&m->address);
     }
@@ -160,7 +159,7 @@ IPAddr::IPAddr(const struct sockaddr *sa, bool unmapv4)
             const uint8_t *bytes =
                 reinterpret_cast<const struct sockaddr_in6 *>(sa)->sin6_addr.s6_addr;
             if (!memcmp(bytes, ipv4mappedprefix, 12)) {
-                struct sockaddr_in *a = reinterpret_cast<struct sockaddr_in*>(m->saddr);
+                auto a = reinterpret_cast<struct sockaddr_in*>(m->saddr);
                 memset(a, 0, sizeof(*a));
                 a->sin_family = AF_INET;
                 memcpy(&a->sin_addr.s_addr, bytes+12, 4);
@@ -178,10 +177,7 @@ IPAddr::IPAddr(const struct sockaddr *sa, bool unmapv4)
     }
 }
 
-IPAddr::~IPAddr()
-{
-    delete m;
-}
+IPAddr::~IPAddr() = default;
 
 bool IPAddr::ok() const
 {
@@ -265,8 +261,8 @@ bool IPAddr::setScopeIdx(const IPAddr& other)
         scopetype() != Scope::LINK || other.scopetype() != Scope::LINK) {
         return false;
     }
-    struct sockaddr_in6 *msa6 = reinterpret_cast<struct sockaddr_in6*>(m->saddr);
-    struct sockaddr_in6 *osa6 = reinterpret_cast<struct sockaddr_in6*>(other.m->saddr);
+    auto msa6 = reinterpret_cast<struct sockaddr_in6*>(m->saddr);
+    auto osa6 = reinterpret_cast<struct sockaddr_in6*>(other.m->saddr);
     msa6->sin6_scope_id = osa6->sin6_scope_id;
     return true;
 }
@@ -279,7 +275,7 @@ std::string IPAddr::straddr() const
 std::string IPAddr::straddr(bool setscope, bool forurl) const
 {
     if (!ok())
-        return std::string();
+        return {};
     
     char buf[200];
     buf[0] = 0;
@@ -290,7 +286,7 @@ std::string IPAddr::straddr(bool setscope, bool forurl) const
     break;
     case AF_INET6:
     {
-        struct sockaddr_in6 *sa6 = reinterpret_cast<struct sockaddr_in6*>(m->saddr);
+        auto sa6 = reinterpret_cast<struct sockaddr_in6*>(m->saddr);
         inet_ntop(m->saddr->sa_family, &sa6->sin6_addr, buf, 200);
         if (!setscope || scopetype() != Scope::LINK) {
             return buf;
@@ -333,18 +329,17 @@ public:
 
 Interface::Interface()
 {
-    m = new Internal;
+    m.reset(new Internal);
 }
 Interface::Interface(const Interface& o)
 {
-    m = new Internal;
+    m.reset(new Internal);
     *m = *(o.m);
 }
 Interface& Interface::operator=(const Interface& o)
 {
     if (&o != this) {
-        delete m;
-        m = new Internal;
+        m.reset(new Internal);
         *m = *(o.m);
     }
     return *this;
@@ -360,10 +355,7 @@ Interface::Interface(const std::string& nm)
 {
     m->name = nm;
 }
-Interface::~Interface()
-{
-    delete m;
-}
+Interface::~Interface() = default;
 
 void Interface::Internal::setflag(Interface::Flags f)
 {
@@ -492,7 +484,6 @@ std::ostream& Interface::print(std::ostream& out) const
 class Interfaces::Internal {
 public:
     Internal();
-    ~Internal() = default;
     std::vector<Interface> interfaces;
 };
 
@@ -746,18 +737,14 @@ out:
 
 Interfaces::Interfaces()
 {
-    m = new Internal();
+    m.reset(new Internal);
 }
 
-Interfaces::~Interfaces()
-{
-    delete m;
-}
+Interfaces::~Interfaces() = default;
 
 bool Interfaces::refresh()
 {
-    delete m;
-    m = new Internal();
+    m.reset(new Internal);
     return true;
 }
 
